#!/usr/bin/env python

"""
Check if a specific ip address and port is listening.

No connection establishment is done during the check.

Uses:
- /proc/net/tcp
- /proc/net/tcp6
"""

import sys
import socket
import struct
assert struct.calcsize('I') == 4

def isLocalTcpPortOpened(ip_address, port):
  family = socket.getaddrinfo(ip_address, 0)[0][0]
  conf = {
    socket.AF_INET6: (4, "/proc/net/tcp6"),
    socket.AF_INET: (1, "/proc/net/tcp"),
  }

  int_count = conf[family][0]
  tcp_path = conf[family][1]

  ip_addr_hex = ('%08X' * int_count) % struct.unpack('I' * int_count, socket.inet_pton(family, ip_address))
  full_addr_hex = ip_addr_hex + ":%04X" % port

  return any(full_addr_hex == line.split()[1] for line in open(tcp_path).readlines())

def main():
  if isLocalTcpPortOpened(sys.argv[1], int(sys.argv[2])):
    return 0
  return 1

if __name__ == "__main__":
  sys.exit(main())



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import argparse
import errno
import gdbm
import json
from lockfile import LockFile
import logging
import logging.handlers
import os
import signal
import socket
import subprocess
import sys
import SocketServer
import StringIO
import threading

# Copied from erp5.util:erp5/util/testnode/ProcessManager.py
def subprocess_capture(p, log, log_prefix, get_output=True):
  def readerthread(input, output, buffer):
    while True:
      data = input.readline()
      if not data:
        break
      if get_output:
        buffer.append(data)
      if log_prefix:
        data = "%s : " % log_prefix +  data
      data = data.rstrip('\n')
      output(data)
  if p.stdout:
    stdout = []
    stdout_thread = threading.Thread(target=readerthread,
                                     args=(p.stdout, log, stdout))
    stdout_thread.daemon = True
    stdout_thread.start()
  if p.stderr:
    stderr = []
    stderr_thread = threading.Thread(target=readerthread,
                                     args=(p.stderr, log, stderr))
    stderr_thread.daemon = True
    stderr_thread.start()
  p.wait()
  if p.stdout:
    stdout_thread.join()
  if p.stderr:
    stderr_thread.join()
  return (p.stdout and ''.join(stdout),
          p.stderr and ''.join(stderr))

class EqueueServer(SocketServer.ThreadingUnixStreamServer):

  daemon_threads = True

  def __init__(self, *args, **kw):
    self.options = kw.pop('equeue_options')
    SocketServer.ThreadingUnixStreamServer.__init__(self,
                                                    RequestHandlerClass=None,
                                                    *args, **kw)
    # Equeue Specific elements
    self.setLogger(self.options.logfile[0], self.options.loglevel[0])
    self.setDB(self.options.database[0])
    if getattr(self.options, 'takeover_triggered_file_path', None):
      self.takeover_triggered_file_path = self.options.takeover_triggered_file_path[0]
    # Lock to only have one command running at the time
    self.thread_lock = threading.Lock()
    # Lockfile is used by other commands to know if an import is ongoing.
    self.lockfile = LockFile(self.options.lockfile)
    self.lockfile.break_lock()

  def setLogger(self, logfile, loglevel):
    self.logger = logging.getLogger("EQueue")
    handler = logging.handlers.WatchedFileHandler(logfile, mode='a')
    # Natively support logrotate
    level = logging._levelNames.get(loglevel, logging.INFO)
    self.logger.setLevel(level)
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    self.logger.addHandler(handler)

  def setDB(self, database):
    self.db = gdbm.open(database, 'cs', 0700)

  def _hasTakeoverBeenTriggered(self):
    if hasattr(self, 'takeover_triggered_file_path') and \
       os.path.exists(self.takeover_triggered_file_path):
      return True
    return False

  def _runCommandIfNeeded(self, command, timestamp):
    with self.thread_lock as thread_lock, self.lockfile as lockfile:
      if self._hasTakeoverBeenTriggered():
        self.logger.info('Takeover has been triggered, preventing to run import script.')
        return
      cmd_list = command.split('\0')
      cmd_readable = ' '.join(cmd_list)
      cmd_executable = cmd_list[0]

      if cmd_executable in self.db and timestamp <= int(self.db[cmd_executable]):
        self.logger.info("%s already run.", cmd_readable)
        return

      self.logger.info("Running %s, %s with output:", cmd_readable, timestamp)
      try:
        sys.stdout.flush()
        p = subprocess.Popen(cmd_list, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        subprocess_capture(p, self.logger.info, '', True)
        if p.returncode == 0:
          self.logger.info("%s finished successfully.", cmd_readable)
          self.db[cmd_executable] = str(timestamp)
        else:
          self.logger.warning("%s exited with status %s." % (cmd_readable, p.returncode))
      except subprocess.CalledProcessError as e:
        self.logger.warning("%s exited with status %s. output is: \n %s" % (
            cmd_readable,
            e.returncode,
            e.output,
        ))

  def process_request_thread(self, request, client_address):
    # Handle request
    self.logger.debug("Connection with file descriptor %d", request.fileno())
    request.settimeout(self.options.timeout)
    request_string = StringIO.StringIO()
    segment = None
    try:
      while segment != '':
        segment = request.recv(1024)
        request_string.write(segment)
    except socket.timeout:
      pass

    command = '127'
    try:
      request_parameters = json.loads(request_string.getvalue())
      timestamp = request_parameters['timestamp']
      command = str(request_parameters['command'])
      self.logger.info("New command %r at %s", command, timestamp)

    except (ValueError, IndexError) :
      self.logger.warning("Error during the unserialization of json "
                          "message of %r file descriptor. The message "
                          "was %r", request.fileno(), request_string.getvalue())

    try:
      request.send(command)
    except:
      self.logger.warning("Couldn't respond to %r", request.fileno())
    self.close_request(request)
    self._runCommandIfNeeded(command, timestamp)
# Well the following function is made for schrodinger's files,
# It will work if the file exists or not
def remove_existing_file(path):
  try:
    os.remove(path)
  except OSError, e:
    if e.errno != errno.ENOENT:
      raise

def main():
  parser = argparse.ArgumentParser(
    description="Run a single threaded execution queue.")
  parser.add_argument('--database', nargs=1, required=True,
                      help="Path to the database where the last "
                      "calls are stored")
  parser.add_argument('--loglevel', nargs=1,
                      default='INFO',
                      choices=[i for i in logging._levelNames
                               if isinstance(i, str)],
                      required=False)
  parser.add_argument('-l', '--logfile', nargs=1, required=True,
                      help="Path to the log file.")
  parser.add_argument('-t', '--timeout', nargs=1, required=False,
                      dest='timeout', type=int, default=3)
  parser.add_argument('--lockfile',
                      help="Path to the lock file created when a command is run")
  parser.add_argument('--takeover-triggered-file-path', nargs=1, required=False,
                      help="Path to the file created by takeover script to state that it has been triggered.")
  parser.add_argument('socket', help="Path to the unix socket")

  args = parser.parse_args()

  socketpath = args.socket

  signal.signal(signal.SIGHUP, lambda *args: sys.exit(-1))
  signal.signal(signal.SIGTERM, lambda *args: sys.exit())

  remove_existing_file(socketpath)
  try:
    server = EqueueServer(socketpath, **{'equeue_options':args})
    server.logger.info("Starting server on %r", socketpath)
    server.serve_forever()
  finally:
    remove_existing_file(socketpath)
    os.kill(0, 9)

if __name__ == '__main__':
  main()



"""Replacement for htpasswd"""
# Original author: Eli Carter
# Taken from http://trac.edgewall.org/browser/trunk/contrib/htpasswd.py
import os
import sys
import random
from optparse import OptionParser

# We need a crypt module, but Windows doesn't have one by default.  Try to find
# one, and tell the user if we can't.
try:
    import crypt
except ImportError:
    try:
        import fcrypt as crypt
    except ImportError:
        sys.stderr.write("Cannot find a crypt module.  "
                         "Possibly http://carey.geek.nz/code/python-fcrypt/\n")
        sys.exit(1)


def salt():
    """Returns a string of 2 randome letters"""
    letters = 'abcdefghijklmnopqrstuvwxyz' \
              'ABCDEFGHIJKLMNOPQRSTUVWXYZ' \
              '0123456789/.'
    return random.choice(letters) + random.choice(letters)


class HtpasswdFile:
    """A class for manipulating htpasswd files."""

    def __init__(self, filename, create=False):
        self.entries = []
        self.filename = filename
        if not create:
            if os.path.exists(self.filename):
                self.load()
            else:
                raise Exception("%s does not exist" % self.filename)

    def load(self):
        """Read the htpasswd file into memory."""
        lines = open(self.filename, 'r').readlines()
        self.entries = []
        for line in lines:
            username, pwhash = line.split(':')
            entry = [username, pwhash.rstrip()]
            self.entries.append(entry)

    def save(self):
        """Write the htpasswd file to disk"""
        open(self.filename, 'w').writelines(["%s:%s\n" % (entry[0], entry[1])
                                             for entry in self.entries])

    def update(self, username, password):
        """Replace the entry for the given user, or add it if new."""
        pwhash = crypt.crypt(password, salt())
        matching_entries = [entry for entry in self.entries
                            if entry[0] == username]
        if matching_entries:
            matching_entries[0][1] = pwhash
        else:
            self.entries.append([username, pwhash])

    def delete(self, username):
        """Remove the entry for the given user."""
        self.entries = [entry for entry in self.entries
                        if entry[0] != username]


def main():
    """%prog [-c] -b filename username password
    Create or update an htpasswd file"""
    # For now, we only care about the use cases that affect tests/functional.py
    parser = OptionParser(usage=main.__doc__)
    parser.add_option('-b', action='store_true', dest='batch', default=False,
                      help='Batch mode; password is passed on the command line IN THE CLEAR.'
                      )
    parser.add_option('-c', action='store_true', dest='create', default=False,
        help='Create a new htpasswd file, overwriting any existing file.')
    parser.add_option('-D', action='store_true', dest='delete_user',
        default=False, help='Remove the given user from the password file.')

    options, args = parser.parse_args()

    def syntax_error(msg):
        """Utility function for displaying fatal error messages with usage
        help.
        """
        sys.stderr.write("Syntax error: " + msg)
        sys.stderr.write(parser.get_usage())
        sys.exit(1)

    if not options.batch:
        syntax_error("Only batch mode is supported\n")

    # Non-option arguments
    if len(args) < 2:
        syntax_error("Insufficient number of arguments.\n")
    filename, username = args[:2]
    if options.delete_user:
        if len(args) != 2:
            syntax_error("Incorrect number of arguments.\n")
        password = None
    else:
        if len(args) != 3:
            syntax_error("Incorrect number of arguments.\n")
        password = args[2]

    passwdfile = HtpasswdFile(filename, create=options.create)

    if options.delete_user:
        passwdfile.delete(username)
    else:
        passwdfile.update(username, password)

    passwdfile.save()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import subprocess
import json
import psutil
import time
from shutil import copyfile
import glob
import argparse
import traceback

# Promise timeout after 20 seconds by default
promise_timeout = 20

def parseArguments():
  """
  Parse arguments for monitor collector instance.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--pid_path',
                      help='Path where the pid of this process will be writen.')
  parser.add_argument('--output',
                      help='The Path of file where Json result of this promise will be saved.')
  parser.add_argument('--promise_script',
                      help='Promise script to execute.')
  parser.add_argument('--promise_folder',
                      help='Folder where to find promises to execute. Hide --promise_script and --promise_name')
  parser.add_argument('--monitor_promise_folder',
                      help='Folder where to find Custom monitor promises to execute.')
  parser.add_argument('--promise_name',
                      help='Title to give to this promise.')
  parser.add_argument('--timeout_file',
                      default='',
                      help='File containing Max timeout for each promise run.')
  parser.add_argument('--promise_type',
                      default='status',
                      help='Type of promise to execute. [status, report].')
  parser.add_argument('--monitor_url',
                      help='Monitor Instance website URL.')
  parser.add_argument('--history_folder',
                      help='Path where old result file will be placed before generate a new json result file.')
  parser.add_argument('--instance_name',
                      default='UNKNOWN Software Instance',
                      help='Software Instance name.')
  parser.add_argument('--hosting_name',
                      default='UNKNOWN Hosting Subscription',
                      help='Hosting Subscription name.')

  return parser

class RunPromise(object):

  def __init__(self, config_parser):
    self.config = config_parser
    self.promise_timeout = promise_timeout
    if self.config.timeout_file and \
            os.path.exists(self.config.timeout_file):
      with open(self.config.timeout_file) as tf:
        timeout = tf.read().strip()
        if timeout.isdigit():
          self.promise_timeout = int(timeout)
        else:
          print "%s it not a valid promise-timeout value" % timeout

  def runpromise(self):

    if self.config.promise_folder:
      # run all promises from the given folder
      return self.runpromise_synchronous()

    if os.path.exists(self.config.pid_path):
      with open(self.config.pid_path, "r") as pidfile:
        try:
          pid = int(pidfile.read(6))
        except ValueError:
          pid = None
        if pid and os.path.exists("/proc/" + str(pid)):
          print("A process is already running with pid " + str(pid))
          return 1
    start_date = ""
    with open(self.config.pid_path, "w") as pidfile:
      process = self.executeCommand(self.config.promise_script)
      ps_process = psutil.Process(process.pid)
      start_date = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(ps_process.create_time()))
      pidfile.write(str(process.pid))

    status_json = self.generateStatusJsonFromProcess(process, start_date=start_date)

    status_json['_links'] = {"monitor": {"href": self.config.monitor_url}}
    status_json['title'] = self.config.promise_name
    status_json['instance'] = self.config.instance_name
    status_json['hosting_subscription'] = self.config.hosting_name
    status_json['type'] = self.config.promise_type

    # Save the lastest status change date (needed for rss)
    status_json['change-time'] = ps_process.create_time()
    if os.path.exists(self.config.output):
      with open(self.config.output) as f:
        try:
          last_result = json.loads(f.read())
          if status_json['status'] == last_result['status'] and last_result.has_key('change-time'):
            status_json['change-time'] = last_result['change-time']
        except ValueError:
          pass

    self.updateStatusHistoryFolder(
      self.config.promise_name,
      self.config.output,
      self.config.history_folder,
      self.config.promise_type
    )
    with open(self.config.output, "w") as outputfile:
      json.dump(status_json, outputfile)
    os.remove(self.config.pid_path)

  def runpromise_synchronous(self):

    if os.path.exists(self.config.pid_path):
      # Check if another run promise is running
      with open(self.config.pid_path) as fpid:
        try:
          pid = int(fpid.read(6))
        except ValueError:
          pid = None
        if pid and os.path.exists("/proc/" + str(pid)):
          print("A process is already running with pid " + str(pid))
          return []

    with open(self.config.pid_path, 'w') as fpid:
      fpid.write(str(os.getpid()))

    promise_folder_list = [self.config.promise_folder]
    if self.config.monitor_promise_folder:
      promise_folder_list.append(self.config.monitor_promise_folder)
    status_list = self.checkPromises(promise_folder_list)
    promises_status_file = os.path.join(self.config.output, '_promise_status')
    previous_state_dict = {}
    new_state_dict = {}
    base_dict = {
      '_links': {"monitor": {"href": self.config.monitor_url}},
      'instance': self.config.instance_name,
      'hosting_subscription': self.config.hosting_name,
    }

    if os.path.exists(promises_status_file):
      with open(promises_status_file) as f:
        try:
          previous_state_dict = json.loads(f.read())
        except ValueError:
          pass

    for status_dict in status_list:
      status_dict.update(base_dict)
      if previous_state_dict.has_key(status_dict['title']):
        status, time = previous_state_dict[status_dict['title']].split('#')
        if status_dict['status'] == status:
          status_dict['change-time'] = float(time)

      promise_result_file = os.path.join(self.config.output, 
                                         "%s.status.json" % status_dict['title'])
      with open(promise_result_file, "w") as outputfile:
        json.dump(status_dict, outputfile)
      new_state_dict[status_dict['title']] = '%s#%s' % (status_dict['status'],
                                                        status_dict['change-time'])
      self.updateStatusHistoryFolder(
        status_dict['title'],
        promise_result_file,
        self.config.history_folder,
        'status'
      )

    with open(promises_status_file, "w") as outputfile:
      json.dump(new_state_dict, outputfile)

    os.remove(self.config.pid_path)

  def updateStatusHistoryFolder(self, name, status_file, history_folder, promise_type):
    history_path = os.path.join(history_folder)
    if not os.path.exists(status_file):
      return
    if not os.path.exists(history_folder):
      return
    if not os.path.exists(history_path):
      try:
        os.makedirs(history_path)
      except OSError, e:
        if e.errno == os.errno.EEXIST and os.path.isdir(history_path):
          pass
        else: raise
    with open(status_file, 'r') as sf:
      try:
        status_dict = json.loads(sf.read())
      except ValueError:
        traceback.print_exc()
        return

    if promise_type == 'status':
      filename = '%s.history.json' % name
      history_file = os.path.join(history_path, filename)
      # Remove links from history (not needed)
      status_dict.pop('_links', None)
      if not os.path.exists(history_file):
        with open(history_file, 'w') as f_history:
          data_dict = {
            "date": time.time(),
            "data": [status_dict]
          }
          f_history.write(json.dumps(data_dict))
      else:
        # Remove useless informations
        status_dict.pop('hosting_subscription', '')
        status_dict.pop('title', '')
        status_dict.pop('instance', '')
        status_dict.pop('type', '')

        with open (history_file, mode="r+") as f_history:
          f_history.seek(0,2)
          position = f_history.tell() -2
          f_history.seek(position)
          #f_history.write(',%s]}' % str(status_dict))
          f_history.write('%s}' % ',{}]'.format(json.dumps(status_dict)))
    elif promise_type == 'report':
      # keep_item_amount = 3
      filename = '%s.history.json' % (
        name)

      copyfile(status_file, os.path.join(history_path, filename))
      """# Don't let history foler grow too much, keep xx files
      file_list = filter(os.path.isfile,
          glob.glob("%s/*.%s.history.json" % (history_path, promise_type))
        )
      file_count = len(file_list)
      if file_count > keep_item_amount:
        file_list.sort(key=lambda x: os.path.getmtime(x))
        while file_count > keep_item_amount:
          to_delete = file_list.pop(0)
          try:
            os.unlink(to_delete)
            file_count -= 1
          except OSError:
            raise"""

  def generateStatusJsonFromProcess(self, process, start_date=None, title=None):
    stdout, stderr = process.communicate()
    status_json = {}
    if process.returncode != 0:
      status_json["status"] = "ERROR"
    else:
      status_json["status"] = "OK"
    if stderr:
      status_json["message"] = stderr
    elif stdout:
      status_json["message"] = stdout
    if start_date:
      status_json["start-date"] = start_date
    if title:
      status_json["title"] = title
    return status_json


  def executeCommand(self, args):
    return subprocess.Popen(
      args,
      #cwd=instance_path,
      #env=None if sys.platform == 'cygwin' else {},
      stdin=None,
      stdout=subprocess.PIPE,
      stderr=subprocess.PIPE
    )

  def checkPromises(self, promise_dir_list):
    """
      Run all promises found into specified folder
    """
    promise_list = []
    for promise_dir in promise_dir_list:
      if not os.path.exists(promise_dir) or not os.path.isdir(promise_dir):
        continue
      promise_list.extend([os.path.join(promise_dir, promise)
                            for promise in os.listdir(promise_dir)])

    promise_result_list = []

    # Check whether every promise is kept
    for promise_script in promise_list:

      if not os.path.isfile(promise_script) or not os.access(promise_script, os.X_OK):
        # Not executable file
        continue

      command = [promise_script]
      promise_name = os.path.basename(command[0])
      result_dict = {
        "status": "ERROR",
        "type": "status",
        "title": promise_name,
        "start-date" : time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())),
        "change-time": time.time()
      }

      process_handler = subprocess.Popen(command,
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE,
                                         stdin=subprocess.PIPE)
      process_handler.stdin.flush()
      process_handler.stdin.close()
      process_handler.stdin = None

      sleep_time = 0.1
      increment_limit = int(self.promise_timeout / sleep_time)
      for current_increment in range(0, increment_limit):
        if process_handler.poll() is None:
          time.sleep(sleep_time)
          continue
        if process_handler.poll() == 0:
          # Success!
          result_dict["message"] = process_handler.communicate()[0]
          result_dict["status"] = "OK"
        else:
          stdout, stderr = process_handler.communicate()
          result_dict["message"] = stderr
          if not stderr:
            result_dict["message"] = stdout
        break
      else:
        process_handler.terminate()
        message = process_handler.stderr.read()
        if message is None:
          message = process_handler.stdout.read() or ""
        message += '\nPROMISE TIME OUT AFTER %s SECONDS' % self.promise_timeout
        result_dict["message"] = message

      promise_result_list.append(result_dict)

    return promise_result_list


def main():
  arg_parser = parseArguments()
  promise_runner = RunPromise(arg_parser.parse_args())
  sys.exit(promise_runner.runpromise())



##############################################################################
#
# Copyright (c) 2015 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import ConfigParser
import argparse
import collections
import json
import logging
import sys
import traceback
import time
import os
import tempfile

import slapos.slap

from slapos.slap.slap import ConnectionError

from slapos.grid.utils import setRunning, setFinished

from erp5.util.taskdistribution import TaskDistributor, TaskDistributionTool
from erp5.util.testnode.Updater import Updater
from erp5.util.testnode.ProcessManager import SubprocessError, ProcessManager, CancellationError



class AutoSTemp(object):
    """
    Create a self-destructing temporary file.
    Uses mkstemp.
    """
    __unlink = os.unlink

    def __init__(self, value):
        fd, self.__name = tempfile.mkstemp()
        os.write(fd, value)
        os.close(fd)

    @property
    def name(self):
        return self.__name

    def __del__(self):
        self.__unlink(self.__name)

from tester import SoftwareReleaseTester

class TestMap(object):
  def __init__(self, test_dict):
    self.ran_test_set = set()
    self.test_map_dict = collections.OrderedDict()
    for key in test_dict:
      group = test_dict[key].get("group", "default")
      if group not in self.test_map_dict:
        self.test_map_dict[group] = [key]

  def addRanTest(self, test):
    self.ran_test_set.add(test)

  def getExcludeList(self, group):
    exclude_list = []
    for key in self.test_map_dict:
      if key != group:
        exclude_list.extend(self.test_map_dict[key])
    return set(exclude_list + list(self.ran_test_set))

  def getGroupList(self):
    return self.test_map_dict.keys()

  def dropGroup(self, group):
    del self.test_map_dict[group]

  def cleanEmptyGroup(self):
    for key in self.test_map_dict.copy():
      if len(self.test_map_dict[key]) == 0:
        del self.test_map_dict[key]

  def getNextGroup(self, ignore_list):
    for group in self.getGroupList():
      if group not in ignore_list:
        return group

    return None

def loadConfiguration(configuration, logger):
  section_dict = collections.OrderedDict()  
  for section in configuration.sections():
    if section == 'agent':
      continue
    section_dict[section] = section_entry_dict = dict(
        configuration.items(section))
    for key in ('request_kw', ):
      if key in section_entry_dict:
        try:
          if isinstance(section_entry_dict[key], str) or \
               isinstance(section_entry_dict[key], unicode):
            section_entry_dict[key] = json.loads(section_entry_dict[key])
        except Exception:
          logger.error("Fail to load %s on %s" % (key, section_entry_dict))
          raise

    if "group" not in section_entry_dict:
      section_entry_dict["group"] = "default"

  return section_dict

def getLogger(log, verbose):
  formatter = logging.Formatter('%(asctime)s %(message)s')
  logger = logging.getLogger()
  if verbose:
    log_level = logging.DEBUG
  else:
    log_level = logging.INFO
  logger.setLevel(log_level)
  handler = logging.StreamHandler(sys.stdout)
  handler.setFormatter(formatter)
  logger.addHandler(handler)
  log_file = None
  if log:
    handler = logging.FileHandler(log)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    log_file = open(log)
    log_file.seek(0, 2)

  return logger, log_file

def getAndUpdateFullRevisionList(node_test_suite, working_directory, 
  logger, process_manager, git_binary="git"):
  full_revision_list = []
  base_path = os.path.join(working_directory, 
    node_test_suite.get('test_suite_reference'))
  for vcs_repository in node_test_suite.get('vcs_repository_list', []):
    repository_path = os.path.join(base_path, 
      vcs_repository['buildout_section_id'])

    repository_id = vcs_repository['buildout_section_id']
    branch = vcs_repository.get('branch')
    # Make sure we have local repository
    updater = Updater(repository_path, git_binary=git_binary,
       branch=branch, log=logger.info, process_manager=process_manager,
       working_directory=working_directory,
       url=vcs_repository["url"])

    retry = 10
    while retry:
      try:
        updater.checkout()
      except SubprocessError:
        # Wait a bit and try again
        time.sleep(30)
        retry -= 1
        continue
      finally:
        break 

    revision = "-".join(updater.getRevision())
    full_revision_list.append('%s=%s' % (repository_id, revision))
  node_test_suite['revision'] = ','.join(full_revision_list)
  return full_revision_list

def main():
  """
  Note: This code does not test as much as it monitors.
  The goal is to regularily try to build & instantiate a software release
  on several machines, to monitor vifib stability and SR stability as time
  passes (and things once available online become unavailable).
  Part of this function could be reused to make an actual test bot, testing
  only when actual changes are committed to a software release, to look for
  regressions.

  Note: This code does not connect to any instantiated service, it relies on
  the presence of a promise section to make instantiation fail until promise
  is happy.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--pidfile', '-p', help='pidfile preventing parallel '
      'execution.')
  parser.add_argument('--log', '-l', help='Log file path.')
  parser.add_argument('--verbose', '-v', help='Be verbose.',
      action='store_true')
  parser.add_argument('configuration_file', type=argparse.FileType(),
      help='Slap Test Agent configuration file.')
  key_file_dict = {}
  args = parser.parse_args()

  log = args.log

  logger, log_file = getLogger(log, args.verbose)

  configuration = ConfigParser.SafeConfigParser()
  configuration.readfp(args.configuration_file)

  pidfile = args.pidfile
  if pidfile:
    setRunning(logger=logger, pidfile=pidfile)
  try:
    while True:

      section_dict = loadConfiguration(configuration, logger)
 
      agent_parameter_dict = dict(configuration.items('agent'))

      task_distributor = TaskDistributor(agent_parameter_dict['report_url'])

      task_distributor.subscribeNode(
          node_title=agent_parameter_dict['node_title'], 
          computer_guid="None")

      test_suite_data = task_distributor.startTestSuite(
          node_title=agent_parameter_dict['node_title'], 
          computer_guid="None")

      task_distribution_tool = TaskDistributionTool(
                       agent_parameter_dict['report_url'],
                       logger=logger)

      if type(test_suite_data) == str:
         # Backward compatiblity
         test_suite_data = json.loads(test_suite_data) 
      
      slap_account_key = task_distributor.getSlaposAccountKey()
      slap_certificate = task_distributor.getSlaposAccountCertificate() 
      master_url = task_distributor.getSlaposUrl()

      key_file_dict = {}
      def asFilenamePair(key, cert):
          # Note: python's ssl support only supports fetching key & cert data
          # from on-disk files. This is why we need to "convert" direct data
          # into file paths, using temporary files.
          cert = cert.strip()
          try:
              temp_key, temp_cert = key_file_dict[cert]
          except KeyError:
              temp_key = AutoSTemp(key.strip())
              temp_cert = AutoSTemp(cert)
              key_file_dict[cert] = (temp_key, temp_cert)
          return temp_key.name, temp_cert.name

      key_file, cert_file = asFilenamePair(slap_account_key, 
        slap_certificate) 


      process_manager = ProcessManager(logger.info)

      for test_suite in test_suite_data:

        full_revision_list = getAndUpdateFullRevisionList(test_suite, 
            agent_parameter_dict["working_directory"], logger, process_manager)
        unit_test_dict = task_distributor.generateConfiguration(
          test_suite['test_suite_title'])

        if not len(full_revision_list):
          # We don't watch git revision but we periodically
          # run the test, once a day.
          full_revision_list = ["day=%s" % time.strftime('%Y/%m/%d', time.gmtime())] 

        if type(unit_test_dict) == str:
          # Backward compatiblity
          unit_test_dict = json.loads(unit_test_dict)
    
        test_result = task_distribution_tool.createTestResult(
          revision=','.join(full_revision_list),
          test_name_list=unit_test_dict.keys(),
          node_title=agent_parameter_dict['node_title'],
          allow_restart=False,
          test_title=test_suite['test_suite_title'],
          project_title=agent_parameter_dict['project_title'],
        )
        if test_result is None:
          # We already have a test result
          logger.info('Skiping test for %s, result already available (%s)' % 
            (test_suite['test_suite_title'], ','.join(full_revision_list)))
          continue

        test_result.watcher_period = 120
        assert test_result is not None
    
        if log_file is not None:
          test_result.addWatch(log, log_file, max_history_bytes=10000)
    
        logger.info("Starting to run for %s" % test_result )
    
        test_mapping = TestMap(unit_test_dict)
        logger.info("Running %s tests in parallel." % \
                      len(test_mapping.getGroupList()))
    
        assert master_url.startswith('https:')
        slap = slapos.slap.slap()
        retry = 0
        while True:
          if retry > 100:
             break
          # wait until _hateoas_navigator is loaded.
          slap.initializeConnection(
            master_url, key_file, cert_file, timeout=120)

          if getattr(slap, '_hateoas_navigator', None) is None:
             logger.info("Fail to load _hateoas_navigator waiting a bit and retry.")
             time.sleep(30)
          else:
             break

        if getattr(slap, '_hateoas_navigator', None) is None:
          raise ValueError("Fail to load _hateoas_navigator")
    
        supply = slap.registerSupply()
        order = slap.registerOpenOrder()
    
        running_test_dict = {}
    
        logger.info('Starting Test Agent run %s ' % agent_parameter_dict['node_title'])
        while True:
          # Get up to parallel_task_count tasks to execute
          while len(running_test_dict) < len(test_mapping.getGroupList())\
                and (len(test_mapping.getGroupList()) > 0):
    
            test_mapping.cleanEmptyGroup()
            
            # Select an unused computer to run the test.
            group = test_mapping.getNextGroup(
              ignore_list = [group for _, _, group in \
                           running_test_dict.itervalues()])
    
            # Select a test 
            test_line = test_result.start(
                exclude_list=list(test_mapping.getExcludeList(group)))
    
            logger.info("Test Line: %s " % test_line)
            logger.info("Ran Test Set: %s " % test_mapping.ran_test_set)
            logger.info("Running test dict: %s " % running_test_dict)
            logger.info("Group: %s " % group)
    
            if test_line is None:
              logger.info("Removing Group (empty test line): %s " % group)
              test_mapping.dropGroup(group)
              continue
    
            test_name = test_line.name
            try:
              section_entry_dict = unit_test_dict[test_name]
            except KeyError:
              # We don't know how to execute this test. Assume it doesn't
              # exist anymore, and fail it in result.
              test_line.stop(stderr='This test does not exist on test '
                  'node %s' % (agent_parameter_dict['node_title'], ))
              continue
    
            general_timeout = agent_parameter_dict.get('timeout', 3600) 
            tester = SoftwareReleaseTester(
                test_name + time.strftime('_%Y/%m/%d_%H:%M:%S_+0000', time.gmtime()),
                logger,
                slap,
                order,
                supply,
                section_entry_dict['url'],
                section_entry_dict.get('supply_computer'),
                section_entry_dict.get('request_kw'),
                agent_parameter_dict.get('software_timeout', general_timeout),
                agent_parameter_dict.get('instance_timeout', general_timeout)
            )
            test_mapping.addRanTest(test_name)
            running_test_dict[test_name] = (test_line, tester, group)
    
          if not running_test_dict:
            logger.info('No more tests to run...')
            break
    
          now = time.time()
          # Synchronise refreshes on watcher period, so it doesn't report a
          # stalled test node where we are actually still sleeping.
          # Change test_result.watcher_period outside this loop if you wish
          # to change sleep duration.
          next_deadline = now + test_result.watcher_period
    
          for section, (test_line, tester, group) in running_test_dict.items():
            logger.info('Checking %s: %r...', section, tester)
            try:
              deadline = tester.tic(now)
            except ConnectionError:
              logger.exception('Test execution ConnectionError for  %s' % (section))
              deadline = next_deadline

            except Exception:
              logger.exception('Test execution fail for  %s' % (section))
              test_line.stop(test_count=1, error_count=1, failure_count=0,
                   skip_count=0, command=tester.getInfo(), 
                   stdout=tester.getFormatedLastMessage(), 
                   stderr=traceback.format_exc())
    
              del running_test_dict[section]
              try:
                tester.teardown()
              except slapos.slap.NotFoundError:
                # This exception is ignored because we cannot
                # Teardown if SR URL do not exist.
                logger.exception('Fail and not found')
                pass
              except Exception:
                logger.exception('teardown failed, human assistance needed for cleanup')
                raise
    
            else:
              logger.info('%r' % tester)
              if deadline is None:
                # TODO: report how long each step took.
                logger.info('Test execution finished for  %s' % (section))
                test_line.stop(test_count=1, error_count=0, failure_count=0,
                          skip_count=0, command=tester.getInfo(), stdout=tester.getFormatedLastMessage())
    
                del running_test_dict[section]
                try:
                  pass #tester.teardown()
                except slapos.slap.NotFoundError:
                  # This exception is ignored because we cannot
                  # Teardown if SR URL do not exist.
                  logger.exception('Fail and not found')
                  pass
                except Exception:
                  logger.exception('teardown failed, human assistance needed for cleanup')
                  raise
    
              else:
                next_deadline = min(deadline, next_deadline)
    
          if running_test_dict:
            to_sleep = next_deadline - time.time()
            if to_sleep > 0:
              logger.info('Sleeping %is...', to_sleep)
              time.sleep(to_sleep)
            if not test_result.isAlive():
              for _, tester, computer_id in running_test_dict.itervalues():
                tester.teardown()

      time.sleep(300)    
  finally:
    if pidfile:
        setFinished(pidfile)
    key_file_dict.clear()
  
if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
import os, time
import sys
import re
import shutil
import tempfile
import unittest
import json
from slapos.monitor.monitor_config_write import MonitorConfigWrite

class MonitorConfigDocument(unittest.TestCase):

  def setUp(self):
    self.base_dir = tempfile.mkdtemp()
    self.config_dir = os.path.join(self.base_dir, 'config')
    self.config_path = os.path.join(self.config_dir, 'config.json')
    os.mkdir(self.config_dir)
    self.httpd_passwd = "btouhjng"
    self.file_content = "wjkqelod"
    self.httpd_passwd_bin = os.path.join(self.base_dir, 'htpasswd')
    self.httpd_passwd_script = """#!/bin/sh
echo "htpasswd $@" > %s/monitor-htpasswd
""" % self.base_dir
    self.monitor_https_cors = os.path.join(self.base_dir, 'httpd-cors-template.cfg.in')
    self.parameter_dict = {
      "cors-domain": 
        {
          "gracefull_bin": ["/bin/echo", "restarted"],
          "type": "httpdcors",
          "cors_file": "%s/test-httpd-cors.cfg" % self.base_dir
        },
      "httpd-password":
        {
          "htpasswd": "%s/monitor-htpasswd" % self.base_dir,
          "type": "htpasswd",
          "user": "admin",
          "file": "%s/.httpd_pwd_real" % self.base_dir
        },
      "from-file": 
        {
          "type": "file",
          "file": "%s/content" % self.base_dir
        }
      }

    self.config = [
      {
        "value": "raw content2",
        "key": "",
        "title": "raw-content2"
      },
      {
        "value": "%s" % self.httpd_passwd,
        "key": "httpd-password",
        "title": "httpd-password"
      },
      {
        "value": "%s" % self.file_content,
        "key": "from-file",
        "title": "from-file"
      },
      {
        "value": "",
        "key": "cors-domain",
        "title": "cors-domain"
      },
      {
        "value": "raw content",
        "key": "",
        "title": "raw-value"
      }
    ]
    self.writeContent("%s/test-httpd-cors.cfg" % self.base_dir, "")
    self.writeContent("%s/monitor-htpasswd" % self.base_dir, "")
    self.writeContent("%s/content" % self.base_dir, self.file_content)
    self.writeContent("%s/.httpd_pwd_real" % self.base_dir, self.httpd_passwd)
    self.writeContent(self.httpd_passwd_bin, self.httpd_passwd_script)
    self.writeContent(self.monitor_https_cors, '{% set allow_domain = "|".join(domain.replace(".", "\.").split()) -%}\n'
                                               'SetEnvIf Origin "^http(s)?://(.+\.)?({{ allow_domain }})$" ORIGIN_DOMAIN=$0\n'
                                               'Header always set Access-Control-Allow-Origin "%{ORIGIN_DOMAIN}e" env=ORIGIN_DOMAIN')
    os.chmod(self.httpd_passwd_bin, 0755)

  def tearDown(self):
    if os.path.exists(self.base_dir):
      shutil.rmtree(self.base_dir)

  def writeContent(self, file_path, config):
    with open(file_path, 'w') as cfg:
      cfg.write(config)

  def generate_cors_string(self, cors_domain_list):
    cors_string = ""
    for domain in cors_domain_list:
      if cors_string:
        cors_string += '|'
      cors_string += re.escape(domain)

    cors_string = 'SetEnvIf Origin "^http(s)?://(.+\.)?(%s)$" ORIGIN_DOMAIN=$0\n' % cors_string
    cors_string += 'Header always set Access-Control-Allow-Origin "%{ORIGIN_DOMAIN}e" env=ORIGIN_DOMAIN'
    return cors_string

  def check_config(self):
    config_parameter = os.path.join(self.config_dir, 'config.parameters.json')
    config_parameter_json = json.load(open(config_parameter))
    config_json = json.load(open(self.config_path))

    for config in config_json:
      if config["key"]:
        self.assertTrue(config_parameter_json.has_key(config["key"]))
        parameter = config_parameter_json[config["key"]]
      else:
        continue
      if config["key"] == 'from-file':
        self.assertTrue(os.path.exists(parameter['file']))
        self.assertEqual(config["value"], open(parameter['file']).read())
      elif config["key"] == 'httpd-password':
        http_passwd = "%s/monitor-htpasswd" % self.base_dir
        #XXX where \n bellow come from ?
        command = 'htpasswd -cb %s admin %s%s' % (http_passwd, config["value"], '\n')
        self.assertTrue(os.path.exists(parameter['file']))
        self.assertTrue(os.path.exists(http_passwd))
        self.assertEquals(config["value"], open(parameter['file']).read())
        self.assertEquals(open(http_passwd).read(), command)
      elif config["key"] == 'cors-domain':
        cors_file = "%s/test-httpd-cors.cfg" % self.base_dir
        self.assertTrue(os.path.exists(cors_file))
        cors_string = self.generate_cors_string(config["value"].split())
        self.assertEquals(cors_string, open(cors_file).read())

  def check_cfg_config(self, config_list):
    cfg_output = os.path.join(self.config_dir, 'config.cfg')
    config_cfg = "[public]\n"
    for config in config_list:
      if config['key']:
        config_cfg += '%s = %s\n' % (config['key'], config['value'])
    with open(cfg_output) as cfg:
      self.assertEqual(cfg.read(), config_cfg)

  def test_write_config_default(self):
    self.writeContent(self.config_path, json.dumps(self.config))
    self.writeContent(os.path.join(self.config_dir, 'config.parameters.json'), json.dumps(self.parameter_dict))
    cfg_output = os.path.join(self.config_dir, 'config.cfg')

    instance = MonitorConfigWrite(
      self.config_path,
      self.httpd_passwd_bin,
      cfg_output,
      self.monitor_https_cors)

    result = instance.applyConfigChanges()
    self.assertTrue(os.path.exists(cfg_output))
    # Check result of non raw parameter edition
    self.assertEquals(result, {'cors-domain': True, 'from-file': True, 'httpd-password': True})
    self.check_config()
    self.check_cfg_config(self.config)

  def test_write_config_parts(self):
    # remove cors config
    for element in self.config:
      if element['key'] == "cors-domain":
        element['key'] = ""
    self.parameter_dict.pop("cors-domain")
    self.writeContent(self.config_path, json.dumps(self.config))
    self.writeContent(os.path.join(self.config_dir, 'config.parameters.json'), json.dumps(self.parameter_dict))
    cfg_output = os.path.join(self.config_dir, 'config.cfg')

    instance = MonitorConfigWrite(
      self.config_path,
      self.httpd_passwd_bin,
      cfg_output)

    result = instance.applyConfigChanges()
    self.assertTrue(os.path.exists(cfg_output))
    # Check result of non raw parameter edition
    self.assertEquals(result, {'from-file': True, 'httpd-password': True})
    self.check_config()
    self.check_cfg_config(self.config)

  def test_write_config_edit_values(self):
    self.writeContent(self.config_path, json.dumps(self.config))
    self.writeContent(os.path.join(self.config_dir, 'config.parameters.json'), json.dumps(self.parameter_dict))
    cfg_output = os.path.join(self.config_dir, 'config.cfg')

    instance = MonitorConfigWrite(
      self.config_path,
      self.httpd_passwd_bin,
      cfg_output,
      self.monitor_https_cors)

    result = instance.applyConfigChanges()
    self.assertTrue(os.path.exists(cfg_output))
    self.assertEquals(result, {'cors-domain': True, 'from-file': True, 'httpd-password': True})
    self.check_config()
    self.check_cfg_config(self.config)

    for config in self.config:
      if config["key"] != "":
        config["value"] = "changed.value"
    self.writeContent(self.config_path, json.dumps(self.config))

    result = instance.applyConfigChanges()
    self.assertEquals(result, {'cors-domain': True, 'from-file': True, 'httpd-password': True})
    self.check_config()
    self.check_cfg_config(self.config)

    # Add new domain in cors domain
    for config in self.config:
      if config["key"] != "cors-domain":
        config["value"] = "changed.value new.domain.com"
    self.writeContent(self.config_path, json.dumps(self.config))

    result = instance.applyConfigChanges()
    self.assertEquals(result, {'cors-domain': True, 'from-file': True, 'httpd-password': True})
    self.check_config()
    self.check_cfg_config(self.config)



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2014 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import json
import urllib
import time

from .erp5 import ERP5TestSuite

class ERP5ClusterTestSuite(ERP5TestSuite):
  """
  Run ERP5 (erp5-cluster flavor) inside Slaprunner Resiliency Test.
  Note: requires specific kernel allowing long shebang paths.
  """
  def _setERP5InstanceParameter(self):
    """
    Set inside of slaprunner the instance parameter to use to deploy erp5 instance.
    """
    self._connectToSlaprunner(
        resource='saveParameterXml',
        data='software_type=create-erp5-site&parameter=%3C%3Fxml+version%3D%221.0%22+encoding%3D%22utf-8%22%3F%3E%0A%3Cinstance%3E%0A%3Cparameter+id%3D%22_%22%3E%7B%22zodb-zeo%22%3A+%7B%22backup-periodicity%22%3A+%22minutely%22%7D%2C+%22mariadb%22%3A+%7B%22backup-periodicity%22%3A+%22minutely%22%7D%7D%3C%2Fparameter%3E%0A%3C%2Finstance%3E%0A'
    )

  def _getERP5Url(self):
    """
    Return the backend url of erp5 instance.
    Note: it is not connection parameter of slaprunner,
    but connection parameter of what is inside of webrunner.
    """
    data = self._connectToSlaprunner(
        resource='getConnectionParameter/slappart7'
    )
    url = json.loads(json.loads(data)['_'])['default-v6']
    self.logger.info('Retrieved erp5 url is:\n%s' % url)
    return url

  def _getERP5Password(self):
    data = self._connectToSlaprunner(
        resource='getConnectionParameter/slappart0'
    )
    password = json.loads(json.loads(data)['_'])['inituser-password']
    self.logger.info('Retrieved erp5 password is:\n%s' % password)
    return password

  def _editHAProxyconfiguration(self):
    """
    XXX pure hack.
    haproxy processes don't support long path for sockets.
    Edit haproxy configuration file of erp5 to make it compatible with long paths
    Then restart haproxy.
    """
    self.logger.info('Editing HAProxy configuration...')

    result = self._connectToSlaprunner(
        resource='/getFileContent',
        data='file=runner_workdir%2Finstance%2Fslappart7%2Fetc%2Fhaproxy.cfg'
    )
    file_content = json.loads(result)['result']
    file_content = file_content.replace('var/run/haproxy.sock', 'ha.sock')
    self._connectToSlaprunner(
        resource='/saveFileContent',
        data='file=runner_workdir%%2Finstance%%2Fslappart7%%2Fetc%%2Fhaproxy.cfg&content=%s' % urllib.quote(file_content)
    )

    # Restart HAProxy
    self._connectToSlaprunner(
        resource='/startStopProccess/name/slappart7:haproxy/cmd/STOPPED'
    )

    time.sleep(15)

  def _gitClone(self):
    ERP5TestSuite._gitClone(self)
    self._connectToSlaprunner(
      resource='/newBranch',
      data='project=workspace%2Fslapos&name=erp5-cluster&create=0'
    )

  def _connectToERP5(self, url, data=None, password=None):
    if password is None:
      password = self._getERP5Password()
    return ERP5TestSuite._connectToERP5(self, url, data, password)

  def _createRandomERP5Document(self, password=None):
    if password is None:
      password = self._getERP5Password()

    return ERP5TestSuite._createRandomERP5Document(self, password)

  def _getCreatedERP5Document(self):
    """ Fetch and return content of ERP5 document created above."""
    url = "%s/erp5/getTitle" % self._getERP5Url()
    return self._connectToERP5(url)

def runTestSuite(*args, **kwargs):
  """
  Run ERP5 erp5-cluster Resiliency Test.
  """
  return ERP5ClusterTestSuite(*args, **kwargs).runTestSuite()




#!/usr/bin/env python

import sys
import os
import re
import jinja2
import json
import argparse
import subprocess
from datetime import datetime
import time


def parseArguments():
  """
  Parse arguments for monitor instance.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--config_folder',
                      help='Path where json configuration/document will be read and write')
  parser.add_argument('--htpasswd_bin',
                      help='Path apache htpasswd binary. Needed to write htpasswd file.')
  parser.add_argument('--output_cfg_file',
                      help='Ouput parameters in cfg file.')
  parser.add_argument('--monitor_https_cors',
                      help='Path to the CORS httpd template.')

  return parser.parse_args()

class MonitorConfigWrite(object):

  def __init__(self, config_json_file, htpasswd_bin, output_cfg_file="", monitor_https_cors=""):
    self.config_json_file = config_json_file
    self.output_cfg_file = output_cfg_file
    self.htpasswd_bin = htpasswd_bin
    self.monitor_https_cors = monitor_https_cors

  def _fileWrite(self, file_path, content):
    if os.path.exists(file_path):
      try:
        with open(file_path, 'w') as wf:
          wf.write(content)
          return True
      except OSError, e:
        print "ERROR while writing changes to %s.\n %s" % (file_path, str(e))
    return False

  def _htpasswdWrite(self, htpasswd_bin,  parameter_dict, value):
    if not os.path.exists(parameter_dict['file']):
      return False
    command = [htpasswd_bin, '-cb', parameter_dict['htpasswd'], parameter_dict['user'], value]
    process = subprocess.Popen(
      command,
      stdin=None,
      stdout=subprocess.PIPE,
      stderr=subprocess.PIPE
    )
    result = process.communicate()[0]
    if process.returncode != 0:
      print result
      return False
    with open(parameter_dict['file'], 'w') as pfile:
      pfile.write(value)
    return True

  def _httpdCorsDomainWrite(self, httpd_cors_file, httpd_gracefull_bin, cors_domain):
    cors_string = ""
    cors_domain_list = cors_domain.split()
    old_httpd_cors_file = os.path.join(
      os.path.dirname(httpd_cors_file),
      'prev_%s' % os.path.basename(httpd_cors_file)
    )
    if os.path.exists(old_httpd_cors_file) and os.path.isfile(old_httpd_cors_file):
      try:
        with open(old_httpd_cors_file, 'r') as cors_file:
          if cors_file.read() == cors_domain:
            if os.path.exists(httpd_cors_file) and (os.stat(httpd_cors_file).st_size > 0
              or (cors_domain == "" and os.stat(httpd_cors_file).st_size == 0)):
              # Skip if cors file is not empty
              return True
      except OSError, e:
        print "Failed to open file at %s. \n%s" % (old_httpd_cors_file, str(e))
    try:
      with open(self.monitor_https_cors, 'r') as cors_template:
        template = jinja2.Template(cors_template.read())
        rendered_string = template.render(domain=cors_domain)
      with open(httpd_cors_file, 'w') as file:
        file.write(rendered_string)
    except OSError, e:
      print "ERROR while writing CORS changes to %s.\n %s" % (httpd_cors_file, str(e))
      return False

    # Save current cors domain list
    try:
      with open(old_httpd_cors_file, 'w') as cors_file:
        cors_file.write(cors_domain)
    except OSError, e:
      print "Failed to open file at %s. \n%s" % (old_httpd_cors_file, str(e))
      return False

    # Restart httpd process
    try:
      subprocess.call(httpd_gracefull_bin)
    except OSError, e:
      print "Failed to execute command %s.\n %s" % (httpd_gracefull_bin, str(e))
      return False
    return True

  def applyConfigChanges(self):
    parameter_config_file = os.path.join(
      os.path.dirname(self.config_json_file),
      'config.parameters.json'
    )
    if not os.path.exists(self.config_json_file) or not os.path.isfile(self.config_json_file):
      #print "ERROR: Config file doesn't exist... Exiting"
      return {}

    new_parameter_list = []
    parameter_list = []
    description_dict = {}
    result_dict = {}

    try:
      with open(self.config_json_file) as tmpfile:
        new_parameter_list = json.loads(tmpfile.read())
    except ValueError:
      print "Error: Couldn't parse json file %s" % self.config_json_file

    with open(parameter_config_file) as tmpfile:
      description_dict = json.loads(tmpfile.read())

    for i in range(0, len(new_parameter_list)):
      key = new_parameter_list[i]['key']
      if key != '':
        description_entry = description_dict[key]
        if description_entry['type'] == 'file':
          result_dict[key] = self._fileWrite(
            description_entry['file'],
            new_parameter_list[i]['value']
          )
        elif description_entry['type'] == 'htpasswd':
          result_dict[key] = self._htpasswdWrite(
            self.htpasswd_bin,
            description_entry,
            new_parameter_list[i]['value']
          )
        elif description_entry['type'] == 'httpdcors':
          result_dict[key] = self._httpdCorsDomainWrite(
            description_entry['cors_file'],
            description_entry['gracefull_bin'],
            new_parameter_list[i]['value']
          )

    if (self.output_cfg_file):
      try:
        with open(self.output_cfg_file, 'w') as pfile:
          pfile.write('[public]\n')
          for parameter in new_parameter_list:
            if parameter['key']:
              pfile.write('%s = %s\n' % (parameter['key'], parameter['value']))
      except OSError, e:
        print "Error failed to create file %s" % self.output_cfg_file
        pass

    return result_dict


def main():
  parser = parseArguments()
  parameter_tmp_file = os.path.join(parser.config_folder, 'config.tmp.json')
  config_file = os.path.join(parser.config_folder, 'config.json')

  # Run 4 times with sleep
  run_counter = 1
  max_runn = 4
  sleep_time = 15

  instance = MonitorConfigWrite(
    parameter_tmp_file,
    parser.htpasswd_bin,
    parser.output_cfg_file,
    parser.monitor_https_cors)

  while True:
    result_dict = instance.applyConfigChanges()
    if result_dict != {}:
      status = True
      for key in result_dict:
        if not result_dict[key]:
          status = False
    
      if status and os.path.exists(parameter_tmp_file):
        try:
          os.unlink(config_file)
        except OSError, e:
          print "ERROR cannot remove file: %s" % parameter_tmp_file
        else:
          os.rename(parameter_tmp_file, config_file)
    if run_counter == max_runn:
      break
    else:
      run_counter += 1
      time.sleep(sleep_time)



from slapos.cloudmgr.lib import getDriverInstance, getNode
from libcloud.types import NodeState
import time
import sys

def getpubliciplist(key, secret, service, node_uuid):
  """Gets public ip(s), can wait for the first IP to appear"""
  public_ip_list = []
  for i in range(100):
    driver = getDriverInstance(service, key, secret)
    node = getNode(driver, node_uuid)
    if node.state not in [NodeState.RUNNING, NodeState.REBOOTING,
        NodeState.PENDING]:
      break
    if node.public_ip[0]:
      public_ip_list = node.public_ip
      break
    else:
      time.sleep(5)

  return dict(
    public_ip_list = public_ip_list
  )

def main():
  print getpubliciplist(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])



import datetime
import feedparser
import time
import unittest

import PyRSS2Gen as RSS2

from slapos.checkfeedaspromise import checkFeedAsPromise

class Option(dict):
    def __init__(self, **kw): 
        self.__dict__.update(kw)
    def __setitem__(i, y):
        self.__dict__[i] = y

class TestCheckFeedAsPromise(unittest.TestCase):
  def getOptionObject(self, **kw):
    """
    Returns an object containing options as properties, to simulate a call
    to the tested script
    """
    option = {
      'title': False,
      'description': False,
      'time_buffer': 0,
      'ok_pattern_list': [],
      'ko_pattern_list': [],
    }
    option.update(kw)
    return Option(**option)


  def generateFeed(self, item_list):
    return RSS2.RSS2(
      title="Feed Title",
      link="http://exemple.com",
      description="Feed Description",
      items=[RSS2.RSSItem(**item) for item in item_list]
    ).to_xml()


  def generateOKFeed(self, extra_item_list=None):
    item_list = [{
      'title': 'Doing Something',
      'description': 'work work work',
      'pubDate': datetime.datetime.now(),
      }, {
      'title': 'Something Finished: OK',
      'description': 'OK FINISHED DONE BASTA',
      'pubDate': datetime.datetime.now(),
    }]
    if isinstance(extra_item_list, list):
      item_list.append(extra_item_list)
    return self.generateFeed(item_list)


  def generateKOFeed(self, extra_item_list=None):
    item_list = [{
      'title': 'Doing Something',
      'description': 'work work work',
      'pubDate': datetime.datetime.now(),
      }, {
      'title': 'Something Finished: Error',
      'description': 'FAILURE oops Arghh',
      'pubDate': datetime.datetime.now(),
    }]
    if isinstance(extra_item_list, list):
      item_list.extend(extra_item_list)
    return self.generateFeed(item_list)


  def test_ifOKFoundNoErrorReturned(self):
    option = self.getOptionObject()
    option.title = True
    feed = self.generateOKFeed()

    option.ok_pattern_list = ['OK']
    self.assertEquals(checkFeedAsPromise(feed, option), "")
    option.title, option.description  = False, True
    option.ok_pattern_list = ['DONE', 'OK']
    self.assertEquals(checkFeedAsPromise(feed, option), "")


  def test_ifKOFoundErrorReturned(self):
    option = self.getOptionObject()
    option.title = True
    feed = self.generateKOFeed()

    option.ko_pattern_list = ['Error']
    self.assertNotEquals(checkFeedAsPromise(feed, option), "")
    option.title, option.description  = False, True
    option.ko_pattern_list = ['FAILURE', 'Error']
    self.assertNotEquals(checkFeedAsPromise(feed, option), "")


  def test_ifNoOKPatternFoundErrorIsRaised(self):
    option = self.getOptionObject()
    option.title = True
    feed = self.generateKOFeed()

    # If no time buffer, then not OK is always wrong
    option.ok_pattern_list = ['OK']
    self.assertNotEquals(len(checkFeedAsPromise(feed, option)), 0)

    # if time buffer, then not OK is wrong only after buffer expires
    extra_item = {
      'title': 'Something is Starting',
      'description': 'Very long operation, but should last less than 1h',
      'pubDate': datetime.datetime.now() - datetime.timedelta(seconds=3600),
    }
    feed = self.generateKOFeed([extra_item,])
    option.time_buffer = 4000
    # buffer longer than last item's age
    self.assertEquals(checkFeedAsPromise(feed, option), "")

    # shorter buffer, we want to raise an error
    option.time_buffer = 1800
    self.assertNotEquals(len(checkFeedAsPromise(feed, option)), 0)


  def test_noItemInTheFeedIsNotAnError(self):
    option = self.getOptionObject()
    option.title = True
    
    feed = self.generateFeed([])
    self.assertEquals(checkFeedAsPromise(feed, option), "")


if __name__ == '__main__':
  unittest.main()



from getprivatekey import getprivatekey
from libcloud.types import NodeState
from slapos.cloudmgr.lib import getDriverInstance, getSSHConnection, getNode
from paramiko import SSHException
import sys

def stop(key, secret, service, node_uuid, ssh_key=None):
  """Stops node"""
  driver = getDriverInstance(service, key, secret)
  node = getNode(driver, node_uuid)
  # Checks state
  if node.state not in [NodeState.RUNNING, NodeState.REBOOTING,
      NodeState.PENDING]:
    return False
  # Good state : connects
  if ssh_key is None:
    ssh_key = getprivatekey(key, secret, service, node_uuid)
  public_ip = node.public_ip[0]
  if not public_ip:
    raise Exception('Node is started but has no IP.')
  try:
    ssh = getSSHConnection(driver, public_ip, ssh_key)
    print('Stopping instance...')
    stdin, stdout, stderr = ssh.exec_command('halt')
  except SSHException, e:
    print('unable to stop')
    raise e
  error_log = stderr.read()
  if error_log:
    raise Exception('''Unable to stop : error log is : 
%r
output is : %r''' % (error_log, stdout.read()))
  return True

def main():
  print stop(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])



# Original work by Cedric de Saint Martin, adapted by Lukasz Nowak
import sys
from slapos.cloudmgr.lib import getDriverInstance

def nodelist(key, secret, service):
  driver = getDriverInstance(service, key, secret)
  node_list = driver.list_nodes()
  return node_list

def uuidlist(key, secret, service):
  return [q.uuid for q in nodelist(key, secret, service)]

def main():
  node_list = nodelist(*sys.argv[1:])
  print 'Available nodes (%s):' % len(node_list)
  for node in node_list:
    print node
